package com.example.bugbattle_sdk;

import android.app.Application;
import android.graphics.Bitmap;

import androidx.annotation.NonNull;

import org.json.JSONObject;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;


import bugbattle.io.bugbattle.BugBattle;
import bugbattle.io.bugbattle.GetBitmapCallback;
import bugbattle.io.bugbattle.controller.BugBattleActivationMethod;
import bugbattle.io.bugbattle.controller.BugBattleNotInitialisedException;
import bugbattle.io.bugbattle.model.APPLICATIONTYPE;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;


/**
 * BugbattleSdkPlugin
 */
public class BugbattleSdkPlugin implements FlutterPlugin, MethodCallHandler {
    private MethodChannel channel;
    private static Application application;

    @Override
    public void onAttachedToEngine(@NonNull final FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "bugbattle_sdk");
        channel.setMethodCallHandler(this);
        application = (Application) flutterPluginBinding.getApplicationContext();
        BugBattle.setBitmapCallback(new GetBitmapCallback() {
            @Override
            public Bitmap getBitmap() {
                return flutterPluginBinding.getFlutterEngine().getRenderer().getBitmap();
            }
        });
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case "initWithToken":
                BugBattle.setApplicationType(APPLICATIONTYPE.FLUTTER);
                if (call.argument("activationMethod").equals("SHAKE")) {
                    BugBattle.initialise((String) call.argument("apiKey"), BugBattleActivationMethod.SHAKE, application);
                } else if (call.argument("activationMethod").equals("SCREENSHOT")) {
                    BugBattle.initialise((String) call.argument("apiKey"), BugBattleActivationMethod.SCREENSHOT, application);
                } else {
                    BugBattle.initialise((String) call.argument("apiKey"), BugBattleActivationMethod.NONE, application);
                }
                break;
            case "initializeMany":
                List<BugBattleActivationMethod> activationMethodsList = new LinkedList<>();
                List<String> activationMethodArgs = call.argument("activationMethods");

                for (int i = 0; i < activationMethodArgs.size(); i++) {
                    if (activationMethodArgs.get(i).equals("SHAKE")) {
                        activationMethodsList.add(BugBattleActivationMethod.SHAKE);
                    } else if (activationMethodArgs.get(i).equals("SCREENSHOT")) {
                        activationMethodsList.add(BugBattleActivationMethod.SCREENSHOT);
                    } else {
                        activationMethodsList.add(BugBattleActivationMethod.NONE);
                    }
                }

                BugBattle.initialise((String) call.argument("apiKey"), activationMethodsList.toArray(new BugBattleActivationMethod[activationMethodsList.size()]), application);
                break;
            case "startBugReporting":
                try {
                    BugBattle.startBugReporting();
                } catch (BugBattleNotInitialisedException ex) {
                    System.out.println(ex);
                }
                break;
            case "enableReplays":
                BugBattle.enableReplay();
                break;
            case "setCustomerEmail":
                BugBattle.setCustomerEmail((String) call.argument("email"));
                break;
            case "attachCustomData":
                JSONObject json = new JSONObject((Map) call.argument("customData"));
                BugBattle.attachCustomData(json);
                break;
            case "enablePrivacyPolicy":
                BugBattle.enablePrivacyPolicy((boolean) call.argument("enable"));
                break;
            case "setPrivacyPolicyUrl":
                BugBattle.setPrivacyPolicyUrl((String) call.argument("url"));
                break;
            case "setApiUrl":
                BugBattle.setApiURL((String) call.argument("url"));
                break;
            case "setLanguage":
                BugBattle.setLanguage((String) call.argument("language"));
                break;
            case "sendSilentBugReport":
                String prio = (String) call.argument("priority");
                BugBattle.SEVERITY severity = BugBattle.SEVERITY.LOW;
                if(prio == "MEDIUM"){
                    severity = BugBattle.SEVERITY.MIDDLE;
                }
                if(prio == "HIGH"){
                    severity = BugBattle.SEVERITY.HIGH;
                }
                BugBattle.sendSilentBugReport((String) call.argument("email"),(String) call.argument("description"),severity);
                break;
            default:
                result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }
}
